#version 330

struct DirectionalLight {
	vec3 directionLight;
	vec3 ambientLight;
	vec3 diffuseLight;
	vec3 specularLight;
};

in vec2 UV;
in vec3 Normals;
in vec3 Position_worldspace;
in vec3 Position_fragment;
in vec4 LightSpacePosition_fragment;

out vec4 color;

uniform int flgIsSky;

uniform vec3 positionCamera;
uniform float shininess;
uniform DirectionalLight directionalLight;

uniform sampler2D myTextureSampler;
uniform sampler2D shadowMap;

float ShadowCalculation(vec4 LightSpacePosition_fragment)
{
    /*vec3 projCoords = (LightSpacePosition_fragment.xyz / LightSpacePosition_fragment.w)  * 0.5 + 0.5;
   
    float nearest = texture(shadowMap, projCoords.xy).r; 
    float current = projCoords.z;
	
    float shadow;
	if(current > nearest){
		shadow = 1.0;
	}else{
		shadow = 0.0;
	}*/

	float shadow = 0.0;
	if(-100<Position_worldspace.x && Position_worldspace.x<100 && -100<Position_worldspace.z && Position_worldspace.z<100){
	vec3 projCoords = (LightSpacePosition_fragment.xyz / LightSpacePosition_fragment.w)  * 0.5 + 0.5;
	float current = projCoords.z;
	vec2 texel = 1.0 / textureSize(shadowMap, 0);
	float bias = 0.005;
	

	for(int x = -1; x <= 1; ++x){
		for(int y = -1; y <= 1; ++y){
			float pcf = texture(shadowMap, projCoords.xy + vec2(x,y) * texel).r;
			shadow += current - bias > pcf ? 1.0 : 0.0;
		}
	}
	shadow /= 9.0;

	}else{
	shadow=0.0;
	}
	
	

    return shadow;
}

void main(){

	vec3 n = normalize(Normals);
	vec3 v = normalize(positionCamera - Position_fragment);
	vec3 l = normalize(directionalLight.directionLight);

	float ln = max(0, dot(n,l));

	vec3 h = normalize(v+l);

	float hn = pow(max(0, dot(n,h)), shininess);

	vec3 ambientColor = directionalLight.ambientLight;
	vec3 diffuseColor = directionalLight.diffuseLight * ln;
	vec3 specularColor = directionalLight.specularLight * hn;

	float shadow = ShadowCalculation(LightSpacePosition_fragment);

	vec3 calculatedColor = (ambientColor + (1.0 - shadow)) * (diffuseColor + specularColor);
	
	color = vec4(texture2D(myTextureSampler, UV).rgba) * vec4(calculatedColor, 1);
	//color = vec4(n, 1.0f);
}