#version 330 core
in vec2 UV;
in vec3 Position_worldspace;
in vec3 Position_fragment;
in vec3 Normals;
in vec3 vertexNormal_cameraspace;
in vec3 vertexTangent_cameraspace;
in vec3 vertexBitangent_cameraspace;
//in vec3 LightDirection_tangentspace;
in mat3 TBN_Out;
in vec3 EyeDirection_tangentspace;
in vec3 _positionCamera;

out vec4 color;

uniform int flgNormalMappingActive;

//spl - Spotlight
uniform int splCount_uf; //Anzahl der SPotlight		
uniform int[32] splActive_uf; //Activ 0 = false sonst true
uniform float[32] splRadiusF_uf;
uniform float[32] splIntensityF_uf;
uniform vec3[32] splPosVec3_uf;
uniform vec3[32] splNormalVec3_uf;
uniform mat4[32] splModelMat4_uf;
uniform mat4[32] splWorldMat4_uf;		

uniform int flgPass;	

uniform sampler2D myTextureSampler;
uniform sampler2D myNormalTextureSampler;
uniform sampler2D myBloomSampler;

uniform vec2 screenSize;
uniform float shininess;

struct DirectionalLight {
	vec3 directionLight;
	vec3 ambientLight;
	vec3 diffuseLight;
	vec3 specularLight;
};

uniform DirectionalLight directionalLight;


void main()
{
	vec2 xA, xB, xC, xD, xE;
	vec3 yA, yB, yC, yD, yE;
	vec3 gaussColor;

	if (flgPass == 0)
	{
		color =  vec4(texture2D(myTextureSampler, UV).rgba);		
		//color =  vec4(0,1,1,1);
		return;
	}
	if (flgPass == 1)
	{
		xA = vec2(((gl_FragCoord.x - 10)/ screenSize.x), ((gl_FragCoord.y - 0) / screenSize.y));
		xB = vec2(((gl_FragCoord.x - 5)/ screenSize.x), ((gl_FragCoord.y - 0) / screenSize.y));
		xC = vec2(((gl_FragCoord.x - 0)/ screenSize.x), ((gl_FragCoord.y - 0) / screenSize.y));
		xD = vec2(((gl_FragCoord.x + 5)/ screenSize.x), ((gl_FragCoord.y + 0) / screenSize.y));
		xE = vec2(((gl_FragCoord.x + 10)/ screenSize.x), ((gl_FragCoord.y + 0) / screenSize.y));
		
		yA = texture2D(myBloomSampler, xA).rgb * 0.061;
		yB = texture2D(myBloomSampler, xB).rgb * 0.242;
		yC = texture2D(myBloomSampler, xC).rgb * 0.383;
		yD = texture2D(myBloomSampler, xD).rgb * 0.242;
		yE = texture2D(myBloomSampler, xE).rgb * 0.061;
		gaussColor = yA + yB + yC + yD + yE;
		color =  vec4(gaussColor, 1);
		//color =  vec4(0,1,1,1);
		return;
	}
	if (flgPass == 2)
	{
		xA = vec2(((gl_FragCoord.x - 0)/ screenSize.x), ((gl_FragCoord.y - 10) / screenSize.y));
		xB = vec2(((gl_FragCoord.x - 0)/ screenSize.x), ((gl_FragCoord.y - 5) / screenSize.y));
		xC = vec2(((gl_FragCoord.x - 0)/ screenSize.x), ((gl_FragCoord.y - 0) / screenSize.y));
		xD = vec2(((gl_FragCoord.x + 0)/ screenSize.x), ((gl_FragCoord.y + 5) / screenSize.y));
		xE = vec2(((gl_FragCoord.x + 0)/ screenSize.x), ((gl_FragCoord.y + 10) / screenSize.y));
		
		yA = texture2D(myBloomSampler, xA).rgb * 0.061;
		yB = texture2D(myBloomSampler, xB).rgb * 0.242;
		yC = texture2D(myBloomSampler, xC).rgb * 0.383;
		yD = texture2D(myBloomSampler, xD).rgb * 0.242;
		yE = texture2D(myBloomSampler, xE).rgb * 0.061;
		gaussColor = yA + yB + yC + yD + yE;		
	}


	vec3 _v,l, _n;
	vec3 ambientIntensitaet = vec3(0.3f, 0.3f, 0.3f);
	float shininess = 15.f;
	vec3 LightColor = vec3(0.0f, 0.0f, 0.0f);

	if (flgNormalMappingActive != 0)
	{
		vec3 TextureNormal_tangentspace = texture( myNormalTextureSampler, UV ).rgb*2.0 - 1.0;	
		_v = normalize(TBN_Out * (_positionCamera - Position_fragment));		
		_n = TextureNormal_tangentspace;
	}
	else
	{
		//_v = normalize(vec3(10,5,6)); //mit oder ohne Kamera Position?
		_v = normalize(_positionCamera-Position_fragment); //mit oder ohne Kamera Position?
		_n = normalize(Normals);	
	}

	
	for (int cnt = 0; cnt < splCount_uf; cnt++)	
	{	
		float spotlightIntensity = 0.f;
		vec3 n;
		vec3 p;
		//Spotlight Intensitt
		if (0 != splActive_uf[cnt])
		{
			//homogene Koordinate ignorieren??
			n = vec3(splWorldMat4_uf[cnt] * splModelMat4_uf[cnt] * vec4(splNormalVec3_uf[cnt],1));
			p = vec3(splWorldMat4_uf[cnt] * vec4(splPosVec3_uf[cnt],1));
			
			if (flgNormalMappingActive != 0)
			{
				l = normalize(TBN_Out * (p - Position_fragment));
			}
			else
			{
				//l = normalize(vec3(2.,3.,5.));
				l = normalize(p - Position_fragment);//frs erste mal fix, spter nchstnheres Spotlight
			}

			float ln = max(0, dot(_n,l));
			vec3 h = normalize(_v+l);
			float hn = pow(max(0, dot(_n,h)), shininess);

			float factor = 0.35;
			
			vec3 diffuseIntensitaet = vec3(1.f, 1.f, 1.f) * ln * factor;
			vec3 specularIntensitaet = vec3(1.f, 1.f, 1.f) * hn * factor;
			
			//Oberflchennormale der zu bestrahlenden Flche, weist in die selbe richtung wie die Spotlightnormale
			//if (dot(n, _n) <= -0.1)
			{
				//Spotlight Strahlrichtung Berechnung
				if (dot(Position_fragment, n) > dot(p, n))
				{
					//Punkt liegt vor dem Spotlight, also in Richtung der Normale				
				
					//Lichtreichweite ist von Position der nicht normalisierten Normale Abhngig
					if (dot(Position_fragment, n) < dot(p + n, n))
					{
						//Punkt liegt vor dem Spotlight, und innerhalb der Licht-Reichweite abhngig vom Normalenvektor
						vec3 n_norm = n * (1.f/ length(n));
						vec3 v = p - Position_fragment;
						float distance_to_plane = abs(dot(v,n_norm));
						float distance_to_plane_max = abs(dot(n,n_norm));
						float factor = 1.f/distance_to_plane_max;
					
						//maximaler radius am ende der reichweite, geringster radius bei der Lichtquelle
						float maximum_distance_from_axis = splRadiusF_uf[cnt] * factor  * distance_to_plane;
					
						//TODO distanz von der normalen(Gerade) zum Punkt, ist der abstand von der zentralen Achse der Lichtabstrahlung
						//http://en.wikipedia.org/wiki/Distance_from_a_point_to_a_line#Vector_formulation
						float distance_to_light_axis = length((v) - dot((v),n_norm) * n_norm);
						//if (dot(n, _n) <= 0)
						{
						if (maximum_distance_from_axis >= distance_to_light_axis)
						{						
							//maximale mgliche Distanz entspricht Rand des Radius, entfernt vom Ende der Normalen im Rechten Winkel
							float maxDistanceFromSource = sqrt(length(n) * length(n) + splRadiusF_uf[cnt] * splRadiusF_uf[cnt]);
							//Distanz nohct abhngig von Lichtachse sondern Abstrahlungsebene machen
							spotlightIntensity = spotlightIntensity + splIntensityF_uf[cnt] * 
												   0.5f * max(0,cos(length(v)  * (3.1415926535f/maxDistanceFromSource))) *
												   0.5f * max(0,cos(distance_to_light_axis * (3.1415926535f/maximum_distance_from_axis)));
							//frs debugging, ist dann aktiv wenn punkt innerhalb der normale liegt	
							//spotlightIntensity = 150.0;
						}
						}
						/*if (dot(n, _n) >= 0)
						{
							LightColor+= vec3(0.,1.,0);
						}
						else
						{
							LightColor+= vec3(1.,0.,0);
						}	*/				
					}				
				}
			}
			LightColor += spotlightIntensity * vec3(.65,.65,.65) + (diffuseIntensitaet + spotlightIntensity * vec3(.65,.65,.65) * specularIntensitaet);// * diffuseIntensitaet + specularIntensitaet;
		}
	}


	color =  vec4(texture2D(myTextureSampler, UV).rgba) * vec4((max(ambientIntensitaet, LightColor)), 1.0f) + vec4(gaussColor.rgb,0.75f);	
}

