
#version 330 core

in vec2 UV;
in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;
in vec3 LightDirection_cameraspace;

in int bSpotlight1;
in vec4 vecSpotlight1Pos;
in vec4 vecSpotlight1Normal;

// Ouput data
out vec4 color;

uniform sampler2D myTextureSampler;
uniform vec3 LightPosition;
uniform mat4 matModel;
uniform mat4 matView;
uniform int bIsSky;


void main()
{

	//Licht
	vec3 DiffuseColor = texture2D(myTextureSampler, UV).rgb;
	
	if (0 == bIsSky)
	{	
		vec3 n;
		/*
		float spotlightIntensity = 0.f;
		
		vec3 p;
		//Spotlight Intensitt
		if (0 != bSpotlight1)
		{
			//homogene Koordinate ignorieren??
			n = vec3(vecSpotlight1Normal.x,vecSpotlight1Normal.y,vecSpotlight1Normal.z);
			p = vec3(vecSpotlight1Pos.x,vecSpotlight1Pos.y,vecSpotlight1Pos.z);
			if (dot(Position_worldspace, n) > dot(p, n))
			{
				//Punkt liegt vor dem Spotlight, also in Richtung der Normale

				
				
				//Lichtreichweite ist von Position der nicht nomralisierten Normale Abhngig
				p = p + n;
				if (dot(Position_worldspace, n) < dot(p, n))
				{
					//Punkt liegt vor dem Spotlight, und innerhalb der Licht-Reichweite abhngig vom Normalenvektor
					p = p - n;
					vec3 n_norm = n * (1/ length(n));
					vec3 v = p - Position_worldspace;
					float distance_to_plane = abs(dot(v,n_norm));
					float distance_to_plane_max = abs(dot(n,n_norm));
					float factor = 1.f/distance_to_plane_max;
					
					//TODO Radius als uniform bergeben
					float radius = 3.f;
					//maximaler radius am ende der reichweite, geringster radius bei der Lichtquelle
					float maximum_distance_from_axis = radius * factor * distance_to_plane;
					
					//TODO distanz von der normalen(Gerade) zum Punkt, ist der abstand von der zentralen Achse der Lichtabstrahlung
					//http://en.wikipedia.org/wiki/Distance_from_a_point_to_a_line#Vector_formulation
					float distance_to_light_axis = length((p-Position_worldspace) - ((p-Position_worldspace) * n) * n);

					if (maximum_distance_from_axis >= distance_to_light_axis)
					{
						spotlightIntensity = 150.0;
					}
					
					//frs debugging, ist dann aktiv wenn punkt innerhalb der normale liegt
					spotlightIntensity = 150.0;
				}
			}
		}
		*/
		
		//Hintergrundlicht
		vec3 AmbientColor = vec3(0.5, 0.5, 0.5) * DiffuseColor;

		//Glanzpunkte
		vec3 SpecularColor = vec3(0.2, 0.2, 0.2);

		//Diffuse

		//Oberflchennormale
		//vec3 n = normalize(Normal_cameraspace);
		n = normalize(Normal_cameraspace);

		//Richtung zur Lichtquelle
		vec3 l = normalize(LightDirection_cameraspace);

		//Winkel zwischen der Oberflchennormale und der Richtung zur Lichtquelle
		//clamp to 0, weil wenn das Licht hinter dem Object ist, ist dot(n, l) negativ
		float nl = clamp(dot(n, l), 0, 1);

		vec3 LightColor = vec3(1, 1, 1);
		float Intensity = 50.0f;
		
		//SpotlightIntensitt
		//Intensity = Intensity + spotlightIntensity;

		//Distanz zum Licht
		float distance = length(LightPosition - Position_worldspace);

		//Specular

		//Richtung zum Auge
		vec3 v = normalize(EyeDirection_cameraspace);

		//Reflexionsstrahl
		vec3 r = reflect(-l, n);

		//Winkel zwischen der Richtung zum Augen und Reflexionsstrahl
		float vr = clamp(dot(v, r), 0, 1);

		color = vec4(AmbientColor + DiffuseColor * LightColor * Intensity * nl / (distance*distance) + SpecularColor * LightColor * Intensity * pow(vr, 4) / (distance*distance), 1.0);
		
	}
	else
	{
		//Himmel wird unabhngig von der "globalen Lichtquelle beleuchtet";
		color = vec4(DiffuseColor,1.0);
	}
}

